/*for 2.0
https://redbear.github.io/arduino/package_redbear_nRF5x_index.json

*/
#include <nRF5x_BLE_API.h>  

/*for 1.5
#include "ble.h"
*/



     
    #define DEVICE_NAME "SALS Probe"
    
  
#define PACKET_START 0X1B
#define PACKET_END 0X00
#define MSG_TYPE_VALUE 0X01
#define PACKET_LENGTH  5
// packet to send.
//first bit is to start \B1
// second bit is what is in the packet
// 3rd and 4th bit is the payload
// fifth bit is the end to say we got the whole thing.

uint8_t msg_packet[PACKET_LENGTH];

#define DEVICE_ID            0xFF 

BLE                          ble;
 
 /*
 Circuit should be
 Power to VIN 
 Ground to GND
 Power for probe VDD
 A4 is input for probe
 D3 programatic turn off to power
 */
const int PROBE_IN_PIN = A4;// Pin connected to PROBE
const int PROBE_DIGITAL_ON_PIN = D3; // Pin connected to power

//timing stuff
unsigned long lastCurrent = 0;
unsigned long disconnectTime=-1; //-1 for init, 0 for connected,   higher for time
const unsigned long offInterval = 600000;//takes 10 minutes to time out if nothing is going on.
const unsigned long sendInterval = 50;//20 times a second to send

// The uuid of service and characteristics
static const uint8_t service1_uuid[]         = {0x71, 0x3D, 0, 0, 0x50, 0x3E, 0x4C, 0x75, 0xBA, 0x94, 0x31, 0x48, 0xF1, 0x8D, 0x94, 0x1E};
static const uint8_t service1_chars1_uuid[]  = {0x71, 0x3D, 0, 3, 0x50, 0x3E, 0x4C, 0x75, 0xBA, 0x94, 0x31, 0x48, 0xF1, 0x8D, 0x94, 0x1E};
static const uint8_t service1_chars2_uuid[]  = {0x71, 0x3D, 0, 2, 0x50, 0x3E, 0x4C, 0x75, 0xBA, 0x94, 0x31, 0x48, 0xF1, 0x8D, 0x94, 0x1E};
static const uint8_t uart_base_uuid_rev[]    = {0x1E, 0x94, 0x8D, 0xF1, 0x48, 0x31, 0x94, 0xBA, 0x75, 0x4C, 0x3E, 0x50, 0, 0, 0x3D, 0x71};
 
uint8_t chars1_value[PACKET_LENGTH] = {0};
uint8_t chars2_value[PACKET_LENGTH] = {0};
 
// Create characteristic and service
GattCharacteristic  characteristic1(service1_chars1_uuid, chars1_value, 1, PACKET_LENGTH, GattCharacteristic::BLE_GATT_CHAR_PROPERTIES_WRITE | GattCharacteristic::BLE_GATT_CHAR_PROPERTIES_WRITE_WITHOUT_RESPONSE );
GattCharacteristic  characteristic2(service1_chars2_uuid, chars2_value, 1, PACKET_LENGTH, GattCharacteristic::BLE_GATT_CHAR_PROPERTIES_NOTIFY | GattCharacteristic::BLE_GATT_CHAR_PROPERTIES_READ);
GattCharacteristic *uartChars[] = {&characteristic1, &characteristic2};
GattService         uartService(service1_uuid, uartChars, sizeof(uartChars) / sizeof(GattCharacteristic *));
DeviceInformationService *deviceInfo;

void disconnectionCallBack(const Gap::DisconnectionCallbackParams_t *params) {
  ble.startAdvertising();                                                                     
disconnectTime= millis();
}


void connectionCallBack(const Gap::ConnectionCallbackParams_t *params) {
disconnectTime= 0;
lastCurrent = 0;
}

 
void gattServerWriteCallBack(const GattWriteCallbackParams *Handler) {
  //ADD IF NEEDED.
  }
 
void setup() {
  Serial.begin(9600); // open the serial port at 9600 bps:
//stop blinking wthnext line
pinMode(D13,OUTPUT);
digitalWrite(D13, HIGH);
disconnectTime= millis();

  pinMode(PROBE_DIGITAL_ON_PIN , OUTPUT); 
digitalWrite(PROBE_DIGITAL_ON_PIN , HIGH);

  pinMode(PROBE_IN_PIN , INPUT); 


Serial.print("recieving loud and clear\n");
  ble.init();
  ble.onDisconnection(disconnectionCallBack);
    ble.onConnection(connectionCallBack);
  ble.onDataWritten(gattServerWriteCallBack);
  disconnectTime= millis();
  // setup adv_data and srp_data
  ble.accumulateAdvertisingPayload(GapAdvertisingData::BREDR_NOT_SUPPORTED);
//    ble.accumulateAdvertisingPayload(GapAdvertisingData::BREDR_NOT_SUPPORTED | GapAdvertisingData::LE_GENERAL_DISCOVERABLE);
  ble.accumulateAdvertisingPayload(GapAdvertisingData::SHORTENED_LOCAL_NAME,
                                   (const uint8_t *)DEVICE_NAME, sizeof(DEVICE_NAME)-1 );
  ble.accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LIST_128BIT_SERVICE_IDS,
                                   (const uint8_t *)uart_base_uuid_rev, sizeof(uart_base_uuid_rev));
  // set adv_type
  ble.setAdvertisingType(GapAdvertisingParams::ADV_CONNECTABLE_UNDIRECTED);
    // add service
//  deviceInfo = new DeviceInformationService(ble, "APH", "Model2.0", "SN1", "2.0", "Firmware0.1", "soft-rev0.1");
    deviceInfo = new DeviceInformationService(ble, "APH", "Model1.5", "SN1", "1.5", "Firmware0.3", "soft-rev0.1");
  ble.addService(uartService);
  // set device name
  ble.setDeviceName((const uint8_t *)DEVICE_NAME);
  // set tx power,valid values are -40, -20, -16, -12, -8, -4, 0, 4
  ble.setTxPower(4);
  // set adv_interval, 100ms in multiples of 0.625ms.
  ble.setAdvertisingInterval(20);
  // set adv_timeout, in seconds
  ble.setAdvertisingTimeout(0);
   
  ble.startAdvertising();                  	
}

void sendMsgBlue(uint16_t  value){
   value=constrain(value,0,1028);
		msg_packet[0]=PACKET_START;
		msg_packet[1]=MSG_TYPE_VALUE;
		msg_packet[2]= value & 0xff;//LOW BIT
		msg_packet[3]=  (value >> 8); //HIGH BIT
		      ble.updateCharacteristicValue(characteristic2.getValueAttribute().getHandle(), msg_packet, sizeof(msg_packet) );
	}
 
void loop() {
Serial.print("recieving loud and clear 2\n");
                       ble.waitForEvent();  
  unsigned long current ;
    current = millis();
//    if (disconnectTime==0){
if((current - lastCurrent) > sendInterval) {
  // Read in the probe's ADC value
uint16_t probeADC = analogRead(PROBE_IN_PIN);
sendMsgBlue(probeADC);
  lastCurrent=current;
}
// ble.waitForEvent();  
/* }else {
//    if (disconnectTime!=0){
  if  ((current - disconnectTime) >= offInterval) {
    digitalWrite(PROBE_DIGITAL_ON_PIN , LOW);  
	delay(10000);
   }
   }
   */
	delay(20);   
}

